@@ -11,13 +11,13 @@ use tracing_core::field::{Field, Visit};
 
 /// The visitor necessary to record values in GELF format.
 #[derive(Debug)]
-pub struct AdditionalFieldVisitor<'a> {
+pub(crate) struct AdditionalFieldVisitor<'a> {
     object: &'a mut HashMap<Cow<'static, str>, Value>,
 }
 
 impl<'a> AdditionalFieldVisitor<'a> {
     /// Create a new [`AdditionalFieldVisitor`] from a [`Map`].
-    pub fn new(object: &'a mut HashMap<Cow<'static, str>, Value>) -> Self {
+    pub(crate) fn new(object: &'a mut HashMap<Cow<'static, str>, Value>) -> Self {
         AdditionalFieldVisitor { object }
     }
 
@@ -41,7 +41,7 @@ impl<'a> AdditionalFieldVisitor<'a> {
 
 impl<'a> Visit for AdditionalFieldVisitor<'a> {
     fn record_debug(&mut self, field: &Field, value: &dyn fmt::Debug) {
-        let value = format!("{:#?}", value);
+        let value = format!("{:?}", value);
         let field_name = field.name();
         match field_name {
             "version" => self.record_value(field_name, value),
@@ -95,7 +95,7 @@ impl<'a> Visit for AdditionalFieldVisitor<'a> {
 
     fn record_u64(&mut self, field: &Field, value: u64) {
         let field_name = field.name();
-        match field.name() {
+        match field_name {
             // GELF requires version: String
             "version" => self.record_value(field_name, value.to_string()),
             // GELF requires host: String
